/*
 * This file contains the MVPANECLASS window class code for the example player.
 */

#include <stdlib.h>
#include <windows.h>
#include <windowsx.h>
#include <commdlg.h>
#include <string.h>
#include <stdarg.h>
#include "ctl3d.h"

/* MediaView Include Files */
#include <medv13.h>

/* Application Include Files */
#include "topic.h"
#include "player.h"
#include "pane.h"
#include "resrc1.h"

#ifdef WIN32
#include "proto32.h"
#else
#include "proto16.h"
#endif

/* Global Variables for Panes */
HWND hCaptureWnd = 0;                           /* initially un-captured */

/* are we currently selecting? */
int selecting = FALSE;
HWND hSelectCapture = 0;

/*****************************************************************************
 **	FUNCTION: Pane_Register													**
 **	PURPOSE: Register the window class for the Pane							**
 **	COMMENTS:																**
 **		The MVPaneClass contains extra bytes for holding Pane data.			**
 **		See pane.h for details.												**
 *****************************************************************************/
BOOL Pane_Register(HINSTANCE hInstance)
	{
	WNDCLASS wc;

	/* create the Pane class */
	wc.style = CS_DBLCLKS;
	wc.lpfnWndProc = PaneWndProc;
	wc.cbClsExtra = 0;
	wc.cbWndExtra = EXTRA_PANE_SIZE;
	wc.hInstance = hInstance;
  	wc.hIcon = LoadIcon (hInstance, "myicon");
	wc.hCursor = 0;
	wc.hbrBackground = GetStockObject(WHITE_BRUSH); 
	wc.lpszMenuName =  0;
	wc.lpszClassName = MVPANECLASS;
	if (RegisterClass(&wc) == 0)
		return(FALSE);

	return(TRUE);
	}

/*****************************************************************************
 **	FUNCTION: Pane_Create													**
 **	PURPOSE: Create a new Pane.												**
 **	COMMENTS:																**
 **		This call creates a MediaView (MV) and a window for displaying		**
 **		it.																	**
 *****************************************************************************/
HWND Pane_Create(HWND hParentWnd, HINSTANCE hInstance, HTITLE hTitle, int type, int paneFlags, PANEHOTPROC fpHotProc)
	{
	HWND hWnd;
	LPMV lpMV;
	ERR err;

	/* Create the PANE Window */
	hWnd = CreateWindow(
			MVPANECLASS,
			"Pane",
			WS_CHILD|WS_VISIBLE|WS_HSCROLL|WS_VSCROLL,
			CW_USEDEFAULT,
			CW_USEDEFAULT,
			CW_USEDEFAULT,
			CW_USEDEFAULT,
			hParentWnd,
			0,
			hInstance,
			0 );
	if (hWnd == 0)
		return(0);

	/* make it initially invisible without scroll bars */
	ShowWindow(hWnd, SW_HIDE);
	ShowScrollBar(hWnd, SB_BOTH, FALSE);

	/* now Create the MediaView (MV) */
	lpMV = lpMVNew(hTitle, &err);
	if (lpMV == 0)
		return(0);

	/* bind MV to the Window handle */
	if (!fMVSetWindow(lpMV, hWnd, &err))
		return(0);

	/* store away the MV and the type */
	SetPaneMV(hWnd, lpMV);
	SetPaneType(hWnd, type);
	SetPaneFlags(hWnd, paneFlags);
	
	/* if there is no Hotspot callback pointer, use the default */
	if (fpHotProc == 0)
		fpHotProc = Pane_Hotspot;
	SetPaneCallback(hWnd, fpHotProc);

	/*
	 * The Windows GetParent function will make the main window the parent
	 * of the first Popup.  We need the parent to be the base Pane,
	 * so we track all of this outside Windows.
	 */
	SetPaneParent(hWnd, hParentWnd);

	/* allow selection of embedded windows */
	fMVAllowEWSelection(lpMV, TRUE);

	return(hWnd);
	}


/*****************************************************************************
 **	FUNCTION: Pane_Destroy													**
 **	PURPOSE: Destroy the Pane.												**
 **	COMMENTS:																**
 *****************************************************************************/
void Pane_Destroy(HWND hWnd)
	{
	LPMV lpMV = GetPaneMV(hWnd);
	HTHLITE hHits;
	
	if (lpMV)
		{
		hHits = hMVGetHighlights(lpMV);

		/* undo any highlights */
		if (hHits)
			{
			GlobalFree(hHits);
			hHits = 0;
			}
		}

	/* destroy the MV */
	if (lpMV)
		MVDelete(lpMV);

	/* and destroy the window */
	DestroyWindow(hWnd);

	return;
	}

/*****************************************************************************
 **	FUNCTION: PaneWndProc													**
 **	PURPOSE: Process messages sent to the Pane.								**
 **	COMMENTS:																**
 *****************************************************************************/
long WINAPI PaneWndProc(hWnd, message, wParam, lParam)
	HWND hWnd;
	UINT message;
	WPARAM wParam;
	LPARAM lParam;
	{
	PAINTSTRUCT ps;
	HDC hDC;

	switch (message) 
		{

		case WM_SIZE:
			/* if sizing is not suppressed, do the layout */
			if (!(GetPaneFlags(hWnd) & P_NOSIZE))
				Pane_Layout(hWnd, 0);
			break;

		case WM_PAINT:
			/* draw the Pane on the display */
			hDC = BeginPaint (hWnd, &ps);
        	Pane_Draw(hWnd, hDC, &(ps.rcPaint));
			EndPaint (hWnd, &ps);
			break;
			
		default:
			return (DefWindowProc(hWnd, message, wParam, lParam));
		}
	return (0);
	}

/*****************************************************************************
 **	FUNCTION: Pane_Draw														**
 **	PURPOSE: Repaint the pane.												**
 **	COMMENTS:																**
 *****************************************************************************/
int Pane_Draw(HWND hWnd, HDC hDC, LPRECT lpR)
	{
	ERR err;
	RECT rect;
	LPMV lpMV = GetPaneMV(hWnd);

	if (lpMV == 0)
		return(ERR_FAILED);

	if (lpR == NULL)
		{
		GetClientRect (hWnd, &rect);
		lpR = &rect;
		}
	if (!fMVApplyToDC (lpMV, hDC, lpR, &err))
		return(err);

	/* the non-scrolling region needs a border at the bottom */
	if (GetPaneType(hWnd) == NSR_PANE && fMVHasSR(lpMV))
		{
		/* be sure to use the whole window rect, not just the update rect */
		GetClientRect (hWnd, &rect);
		MoveToEx(hDC, rect.left, rect.bottom-1, NULL);
		LineTo(hDC, rect.right, rect.bottom-1);
		}
	return(0);
	}

/*****************************************************************************
 **	FUNCTION: Pane_Layout													**
 **	PURPOSE: Realize the Pane ... this does not actually paint, that		**
 **		is done handling the WM_PAINT message.								**
 **	COMMENTS:																**
 *****************************************************************************/
void Pane_Layout(HWND hWnd, LPRECT lpR)
	{
	LPMV lpMV = GetPaneMV(hWnd);
	ERR err;

	if (lpMV == 0)
		return;

		/* this does the MediaView layout of the subTopic */
		fMVRealize(lpMV, lpR, (LPERR)&err);

	return;
	}







/*****************************************************************************
 **	FUNCTION: Pane_Hotspot													**
 **	PURPOSE: This is the default PANE hotspot handler hotspot handler		**
 **	COMMENTS:																**
 **		This is what is called if the Pane_Open was passed a NULL			**
 **		pointer for the fpHotspotProc argument.								**
 *****************************************************************************/
BOOL CALLBACK Pane_Hotspot(LPMV lpMV, LPMVHOTINFO lpHI)
	{

	if (lpMV == 0)
		return(FALSE);
	
	switch (lpHI->nType)
		{

		case HOTSPOT_HASH:
			break;

		case HOTSPOT_POPUPHASH:
			break;

		case HOTSPOT_STRING:
			/* put string hotspot handling here */
			break;
			
		case HOTSPOT_UNKNOWN:
		default:
			/* handle unknown hotspots here */
			return(FALSE);
			break;
		}
	return(TRUE);
	}
 
/*****************************************************************************
 **	FUNCTION: Pane_SetAddress												**
 **	PURPOSE: Set the MediaView address.										**
 **	COMMENTS:																**
 *****************************************************************************/
int Pane_SetAddress(HWND hWnd, VA va, long scroll)
	{
	ERR err;
	LPMV lpMV = GetPaneMV(hWnd);
	int iSubTopic;

	/* Set the address. We retrieve the subTopic from the hWnd */
	iSubTopic = GetPaneType(hWnd);
	if (!fMVSetAddress(lpMV, va, iSubTopic, scroll, (LPERR)&err))
		{
			return(err);
		}

	/* the display update is done from the calling routine */
	return(0);
	}
 
/*****************************************************************************
 **	FUNCTION: Pane_Popup													**
 **	PURPOSE: manifest a topic in a POPUP window								**
 **	COMMENTS:																**
 **		Popups will cascade. If the mouse is clicked outside an active		**
 **		popup, this code will clean them up back to the window with the		**
 **		click.																**
 *****************************************************************************/
int Pane_Popup(LPMV lpParentMV, LPRECT lpR, VA va)
	{


	return(TRUE);
	}

/*****************************************************************************
 **	FUNCTION: Pane_CloseAllPopups											**
 **	PURPOSE: If any Popups are in effect, close them.						**
 **	COMMENTS:																**
 *****************************************************************************/
HWND Pane_CloseAllPopups(HWND hWnd)
	{
	return(hWnd);
	}



/*****************************************************************************
 **	FUNCTION: Pane_KeyDown													**
 **	PURPOSE: Process the keydown message.									**
 **	COMMENTS:																**
 *****************************************************************************/
void Pane_KeyDown(HWND hWnd, WPARAM wParam, LPARAM lParam)
	{



	}

